@@ -71,8 +71,11 @@ class const_iterator;
 class const_preorder_iterator;
 class const_postorder_iterator;
 class symbol;
-class symbolhasher;
+struct symbolhasher;
 using symbolset = std::unordered_set<symbol,symbolhasher>;
+using expairvec = std::vector<std::pair<ex,ex>>;
+using ocvector = std::vector<numeric>;
+using power_ocvector_map = std::map<ex, ocvector, GiNaC::ex_is_less>;
 
 /** Lightweight wrapper for GiNaC's symbolic objects.  It holds a pointer to
  *  the other object in order to do garbage collection by the method of
@@ -86,9 +89,9 @@ class ex {
 	template<class T> friend inline bool is_a(const ex &);
 	template<class T> friend inline bool is_exactly_a(const ex &);
 	
-	friend struct print_order;
-	friend struct print_order_mul;
-	friend struct print_order_pair;
+	friend class print_order;
+	friend class print_order_mul;
+	friend class print_order_pair;
 	// default constructor, copy constructor and assignment operator
 public:
 	ex() throw();
@@ -148,7 +151,7 @@ public:
 	size_t nsymbols() const;
         bool get_first_symbol(ex &x) const;
         symbolset symbols() const;
-	ex op(size_t i) const { return bp->op(i); }
+	const ex op(size_t i) const { return bp->op(i); }
 	ex sorted_op(size_t i) const;
 	ex operator[](const ex & index) const { return (*bp)[index]; }
 	ex operator[](size_t i) const { return (*bp)[i]; }
@@ -191,6 +194,7 @@ public:
 	ex coeff(const ex & s, int n = 1) const { return bp->coeff(s, n); }
 	ex lcoeff(const ex & s) const { return coeff(s, degree(s)); }
 	ex tcoeff(const ex & s) const { return coeff(s, ldegree(s)); }
+        void coefficients(const ex & s, expairvec & vec) const;
 
 	// expand/collect
 	ex expand(unsigned options=0) const;
@@ -199,9 +203,11 @@ public:
 	// differentiation and series expansion
 	ex diff(const symbol & s, unsigned nth = 1) const;
 	ex series(const ex & r, int order, unsigned options = 0) const;
+        void useries(flint_series_t& fp, int order) const { return bp->useries(fp, order); }
 
 	// rational functions
-	ex normal(int level = 0) const;
+	ex normal(int level = 0, bool noexpand_combined=false,
+                        bool noexpand_numer=true) const;
 	ex to_rational(exmap & repl) const;
 	ex to_rational(lst & repl_lst) const;
 	ex to_polynomial(exmap & repl) const;
@@ -209,9 +215,13 @@ public:
 #ifdef PYNAC_HAVE_LIBGIAC
         const giac::polynome to_polynome(ex_int_map& map, exvector& revmap) const;
 #endif
+        const CanonicalForm to_canonical(ex_int_map& map,
+                        power_ocvector_map& pomap, exvector& revmap) const;
+        void collect_powers(power_ocvector_map& pomap) const;
 	ex numer() const;
 	ex denom() const;
 	ex numer_denom() const;
+        ex combine_fractions(bool deep=true) const;
 
 	// polynomial algorithms
 	ex unit(const ex &x) const;
@@ -263,6 +273,11 @@ private:
 	static ptr<basic> construct_from_string_and_lst(const std::string &s, const ex &l);
 	void makewriteable();
 	void share(const ex & other) const;
+        static ex deep_combine_fractions(ex e);
+        struct combine_map_function : public map_function {
+                ex operator()(const ex & e) override { return deep_combine_fractions(e); }
+        };
+
 
 // member variables
 
@@ -718,8 +733,9 @@ inline ex denom(const ex & thisex)
 inline ex numer_denom(const ex & thisex)
 { return thisex.numer_denom(); }
 
-inline ex normal(const ex & thisex, int level=0)
-{ return thisex.normal(level); }
+inline ex normal(const ex & thisex, int level=0, bool noexpand_combined=false,
+                bool noexpand_numer=true)
+{ return thisex.normal(level, noexpand_combined, noexpand_numer); }
 
 inline ex to_rational(const ex & thisex, lst & repl_lst)
 { return thisex.to_rational(repl_lst); }
